@@ -24,18 +24,35 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-"""
-Reference material:
+"""Reference material:
 
 Prefixes for binary multiples:
 http://physics.nist.gov/cuu/Units/binary.html
 
 decimal and binary prefixes:
 man 7 units (from the Linux Documentation Project 'man-pages' package)
+
+
+BEFORE YOU GET HASTY WITH EXCLUDING CODE FROM COVERAGE: If you
+absolutely need to skip code coverage because of a strange Python 2.x
+vs 3.x thing, use the fancy environment substitution stuff from the
+.coverage RC file. In review:
+
+* If you *NEED* to skip a statement because of Python 2.x issues add the following:
+
+  # pragma: PY2X no cover
+
+* If you *NEED* to skip a statement because of Python 3.x issues add the following:
+
+  # pragma: PY3X no cover
+
+In this configuration, statements which are skipped in 2.x are still
+covered in 3.x, and the reverse holds true for tests skipped in 3.x.
 """
 
 from __future__ import print_function
 
+import argparse
 import contextlib
 import fnmatch
 import math
@@ -52,7 +69,7 @@ __all__ = [
 
 # Python 3.x compat
 if sys.version > '3':
-    long = int
+    long = int  # pragma: PY2X no cover
 
 # Constants for referring to prefix systems
 NIST = int(2)
@@ -82,6 +99,12 @@ NIST_STEPS = {
     'Ei': 1152921504606846976
 }
 
+# A list of all the valid prefix unit types. Mostly for reference, I
+# suppose. May be useful later on down the road, I think?
+ALL_UNIT_TYPES = ['b', 'B', 'kb', 'kB', 'Mb', 'MB', 'Gb', 'GB', 'Tb', 'TB',
+                  'Pb', 'PB', 'Eb', 'EB', 'Kib', 'KiB', 'Mib', 'MiB', 'Gib',
+                  'GiB', 'Tib', 'TiB', 'Pib', 'PiB', 'Eib', 'EiB']
+
 ######################################################################
 # Set up our module variables/constants
 
@@ -102,13 +125,9 @@ format_plural = False
 
 
 ######################################################################
-# First, the bytes...
-
-
-class Byte(object):
-    """The base class for all the other prefix classes
-
-Byte based types fundamentally operate on self._bit_value"""
+# Base class for everything else
+class Bitmath(object):
+    """The base class for all the other prefix classes"""
 
     def __init__(self, value=0, bytes=None, bits=None):
         """Instantiate with `value` by the unit, in plain bytes, or
@@ -168,16 +187,18 @@ converted *to* this unit"""
         return value / float(self._unit_value)
 
     def _setup(self):
-        return (2, 0, 'Byte', 'Bytes')
+        raise NotImplementedError("The base 'bitmath.Bitmath' class can not be used directly")
 
     def _do_setup(self):
-        """Setup basic parameters for this class"""
-        """`base` is the numeric base which when raised to `power` is
-equivalent to 1 unit of the corresponding prefix. I.e., base=2,
-power=10 represents 2^10, which is the NIST Binary Prefix for 1 Kibibyte.
+        """Setup basic parameters for this class.
+
+`base` is the numeric base which when raised to `power` is equivalent
+to 1 unit of the corresponding prefix. I.e., base=2, power=10
+represents 2^10, which is the NIST Binary Prefix for 1 Kibibyte.
 
 Likewise, for the SI prefix classes `base` will be 10, and the `power`
-for the Kilobyte is 3."""
+for the Kilobyte is 3.
+"""
         (self._base, self._power, self._name_singular, self._name_plural) = self._setup()
         self._unit_value = self._base ** self._power
 
@@ -303,7 +324,7 @@ instantiate the class ahead of time.
 >>> print kib
 KiB(1024.0)
 """
-        if isinstance(item, Byte):
+        if isinstance(item, Bitmath):
             return cls(bits=item.bits)
         else:
             raise ValueError("The provided items must be a valid bitmath class: %s" %
@@ -765,7 +786,7 @@ equivalent of the this instances prefix Unix value. That is to say:
 
     def __long__(self):
         """Return this instances prefix unit as a long integer"""
-        return long(self.prefix_value)
+        return long(self.prefix_value)  # pragma: PY3X no cover
 
     def __float__(self):
         """Return this instances prefix unit as a floating point number"""
@@ -834,8 +855,18 @@ bit of x AND of y is 0, otherwise it's 1."""
     #     return NotImplemented
 
 
+######################################################################
+# First, the bytes...
+
+class Byte(Bitmath):
+    """Byte based types fundamentally operate on self._bit_value"""
+    def _setup(self):
+        return (2, 0, 'Byte', 'Bytes')
+
 ######################################################################
 # NIST Prefixes for Byte based types
+
+
 class KiB(Byte):
     def _setup(self):
         return (2, 10, 'KiB', 'KiBs')
@@ -900,7 +931,7 @@ class EB(Byte):
 
 ######################################################################
 # And now the bit types
-class Bit(Byte):
+class Bit(Bitmath):
     """Bit based types fundamentally operate on self._bit_value"""
 
     def _set_prefix_value(self):
@@ -980,6 +1011,58 @@ class Eb(Bit):
         return (10, 18, 'Eb', 'Ebs')
 
 
+######################################################################
+# Integrations with 3rd party modules
+def BitmathType(bmstring):
+    """An 'argument type' for integrations with the argparse module.
+
+For more information, see
+https://docs.python.org/2/library/argparse.html#type Of particular
+interest to us is this bit:
+
+   ``type=`` can take any callable that takes a single string
+   argument and returns the converted value
+
+I.e., ``type`` can be a function (such as this function) or a class
+which implements the ``__call__`` method.
+
+Example usage of the bitmath.BitmathType argparser type:
+
+   >>> import bitmath
+   >>> import argparse
+   >>> parser = argparse.ArgumentParser()
+   >>> parser.add_argument("--file-size", type=bitmath.BitmathType)
+   >>> parser.parse_args("--file-size 1337MiB".split())
+   Namespace(file_size=MiB(1337.0))
+
+Invalid usage includes any input that the bitmath.parse_string
+function already rejects. Additionally, **UNQUOTED** arguments with
+spaces in them are rejected (shlex.split used in the following
+examples to conserve single quotes in the parse_args call):
+
+   >>> parser = argparse.ArgumentParser()
+   >>> parser.add_argument("--file-size", type=bitmath.BitmathType)
+   >>> import shlex
+
+   >>> # The following is ACCEPTABLE USAGE:
+   ...
+   >>> parser.parse_args(shlex.split("--file-size '1337 MiB'"))
+   Namespace(file_size=MiB(1337.0))
+
+   >>> # The following is INCORRECT USAGE because the string "1337 MiB" is not quoted!
+   ...
+   >>> parser.parse_args(shlex.split("--file-size 1337 MiB"))
+   error: argument --file-size: 1337 can not be parsed into a valid bitmath object
+"""
+    try:
+        argvalue = parse_string(bmstring)
+    except ValueError:
+        raise argparse.ArgumentTypeError("'%s' can not be parsed into a valid bitmath object" %
+                                         bmstring)
+    else:
+        return argvalue
+
+
 ######################################################################
 # Utility functions
 def getsize(path, bestprefix=True, system=NIST):
@@ -1086,6 +1169,8 @@ the unit.
         raise ValueError("Can't parse string %s into a bitmath object" % s)
 
 
+######################################################################
+# Contxt Managers
 @contextlib.contextmanager
 def format(fmt_str=None, plural=False, bestprefix=False):
     """Context manager for printing bitmath instances.
@@ -1124,8 +1209,6 @@ def cli_script_main(cli_args):
     """
     A command line interface to basic bitmath operations.
     """
-    import argparse
-
     choices = ['Bit', 'Byte', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'kB',
                'MB', 'GB', 'TB', 'PB', 'EB', 'Kib', 'Mib', 'Gib', 'Tib',
                'Pib', 'Eib', 'kb', 'Mb', 'Gb', 'Tb', 'Pb', 'Eb']
@@ -1172,7 +1255,7 @@ def cli_script_main(cli_args):
 def cli_script():  # pragma: no cover
     # Wrapper around cli_script_main so we can unittest the command
     # line functionality
-    for result in cli_script_main(sys.argv):
+    for result in cli_script_main(sys.argv[1:]):
         print(result)
 
 if __name__ == '__main__':
